@@ -211,12 +211,19 @@ pub mod visit {
     fn visit_error_token(&mut self, error: &'a ErrorToken) {
       walk_error_token(self, error)
     }
+
+    fn should_continue(&self) -> bool {
+      true
+    }
   }
 
   pub fn walk_expression<'a, V: AstVisitor<'a> + ?Sized>(
     visitor: &mut V,
     expression: &'a Expression,
   ) -> () {
+    if !visitor.should_continue() {
+      return;
+    }
     match *expression {
       Expression::AbsolutePath(ref path) => visitor.visit_absolute_path(path),
       Expression::WildcardPath(ref path) => visitor.visit_wildcard_path(path),
@@ -235,7 +242,13 @@ pub mod visit {
     visitor: &mut V,
     unary_expression: &'a UnaryExpression,
   ) -> () {
+    if !visitor.should_continue() {
+      return;
+    }
     visitor.visit_unary_operator(&unary_expression.op);
+    if !visitor.should_continue() {
+      return;
+    }
     visitor.visit_expression(&unary_expression.expr);
   }
 
@@ -243,8 +256,17 @@ pub mod visit {
     visitor: &mut V,
     binary_expression: &'a BinaryExpression,
   ) -> () {
+    if !visitor.should_continue() {
+      return;
+    }
     visitor.visit_expression(&binary_expression.lhs);
+    if !visitor.should_continue() {
+      return;
+    }
     visitor.visit_binary_operator(&binary_expression.op);
+    if !visitor.should_continue() {
+      return;
+    }
     visitor.visit_expression(&binary_expression.rhs);
   }
 
@@ -255,6 +277,9 @@ pub mod visit {
     match *error {
       ErrorToken::List(ref expressions) => {
         for expression in expressions {
+          if !visitor.should_continue() {
+            return;
+          }
           visitor.visit_expression(&expression);
         }
       }
